# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0002_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='BiologicallyDerivedProductCollection',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('collected_date_time', models.DateTimeField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='BiologicallyDerivedProductProperty',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('value_boolean', models.BooleanField(null=True)),
                ('value_integer', models.IntegerField(null=True)),
                ('value_string', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='BiologicallyDerivedProductReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='BiologicallyDerivedProduct',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('division', models.CharField(max_length=255, null=True)),
                ('expiration_date', models.DateTimeField(null=True)),
                ('biological_source_event', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='biologically_derived_products_biological_source_event', to='base.identifier')),
                ('identifier', models.ManyToManyField(blank=True, related_name='biologically_derived_products_identifier', to='base.identifier')),
                ('processing_facility', models.ManyToManyField(blank=True, related_name='biologically_derived_products_processing_faciality', to='base.organizationreference')),
                ('product_category', models.ManyToManyField(blank=True, related_name='biologically_derived_products_product_category', to='base.codeableconcept')),
                ('product_code', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='biologically_derived_products_product_code', to='base.codeableconcept')),
                ('product_status', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='biologically_derived_products_product_status', to='base.coding')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""body structure views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.bodystructures.models import BodyStructure
from dfhir.bodystructures.serializers import BodyStructureSerializer


class BodyStructureListView(APIView):
    """body structure view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: BodyStructureSerializer(many=True)})
    def get(self, request):
        """Get body structures."""
        body_structures = BodyStructure.objects.all()
        serializer = BodyStructureSerializer(body_structures, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=BodyStructureSerializer, responses={201: BodyStructureSerializer}
    )
    def post(self, request):
        """Create body structure."""
        serializer = BodyStructureSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class BodyStructureDetailView(APIView):
    """body structure detail view."""

    def get_object(self, pk):
        """Get object."""
        try:
            return BodyStructure.objects.get(pk=pk)
        except BodyStructure.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: BodyStructureSerializer})
    def get(self, request, pk):
        """Get body structure."""
        body_structure = self.get_object(pk)
        serializer = BodyStructureSerializer(body_structure)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=BodyStructureSerializer, responses={200: BodyStructureSerializer}
    )
    def patch(self, request, pk):
        """Update body structure."""
        body_structure = self.get_object(pk)
        serializer = BodyStructureSerializer(
            body_structure, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk):
        """Delete body structure."""
        body_structure = self.get_object(pk)
        body_structure.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""Device usage serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    PeriodSerializer,
    TimingSerializer,
)
from dfhir.bodystructures.serializers import BodyStructureCodeableReferenceSerializer
from dfhir.devices.serializers import DeviceDeviceDefinitionCodeableReferenceSerializer
from dfhir.encounters.serializers import EncounterEpisodeOfCareReferenceSerializer
from dfhir.patients.serializers import PatientReferenceSerializer
from dfhir.servicerequests.serializers import ServiceRequestReferenceSerializer

from .models import (
    DeviceUsage,
    DeviceUsageAdherence,
    DeviceUsageDerivedFromReference,
    DeviceUsageInformationSourceReference,
    DeviceUsageReasonCodeableReference,
    DeviceUsageReasonReference,
)


class DeviceUsageDerivedFromReferenceSerializer(BaseReferenceModelSerializer):
    """Device usage derived from reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = DeviceUsageDerivedFromReference
        exclude = ["created_at", "updated_at"]


class DeviceUsageReasonReferenceSerializer(BaseReferenceModelSerializer):
    """Device usage reason reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = DeviceUsageReasonReference
        exclude = ["created_at", "updated_at"]


class DeviceUsageReasonCodeableReferenceSerializer(WritableNestedModelSerializer):
    """Device usage reason codeable reference serializer."""

    reference = DeviceUsageReasonReferenceSerializer(many=False, required=False)
    concept = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = DeviceUsageReasonCodeableReference
        exclude = ["created_at", "updated_at"]


class DeviceUsageInformationSourceReferenceSerializer(BaseReferenceModelSerializer):
    """Device usage information source reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = DeviceUsageInformationSourceReference
        exclude = ["created_at", "updated_at"]


class DeviceUsageAdherenceSerializer(WritableNestedModelSerializer):
    """Device usage adherence serializer."""

    code = CodeableConceptSerializer(many=False, required=False)
    reason = CodeableConceptSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = DeviceUsageAdherence
        exclude = ["created_at", "updated_at"]


class DeviceUsageSerializer(BaseWritableNestedModelSerializer):
    """Device usage serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    based_on = ServiceRequestReferenceSerializer(many=False, required=False)
    category = CodeableConceptSerializer(many=True, required=False)
    patient = PatientReferenceSerializer(many=False, required=False)
    derived_from = DeviceUsageDerivedFromReferenceSerializer(many=True, required=False)
    context = EncounterEpisodeOfCareReferenceSerializer(many=False, required=False)
    timing_timing = TimingSerializer(many=False, required=False)
    timing_period = PeriodSerializer(many=False, required=False)
    usage_status = CodeableConceptSerializer(many=False, required=False)
    usage_reason = CodeableConceptSerializer(many=True, required=False)
    adherence = DeviceUsageAdherenceSerializer(many=False, required=False)
    information_source = DeviceUsageInformationSourceReferenceSerializer(
        many=False, required=False
    )
    device = DeviceDeviceDefinitionCodeableReferenceSerializer(
        many=False, required=False
    )
    reason = DeviceUsageReasonCodeableReferenceSerializer(many=True, required=False)
    body_site = BodyStructureCodeableReferenceSerializer(many=False, required=False)
    note = AnnotationSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = DeviceUsage
        exclude = ["created_at", "updated_at"]



"""Explanation of Benefits serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer
from rest_framework import serializers

from dfhir.base.serializers import (
    AddressSerializer,
    AttachmentSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    CodingSerializer,
    IdentifierSerializer,
    MoneySerializer,
    OrganizationReferenceSerializer,
    PeriodSerializer,
    QuantitySerializer,
    ReferenceSerializer,
    SimpleQuantitySerializer,
)
from dfhir.bodystructures.serializers import BodyStructureCodeableReferenceSerializer
from dfhir.claimresponses.serializers import ClaimResponseReferenceSerializer
from dfhir.claims.serializers import ClaimReferenceSerializer
from dfhir.conditions.serializers import ConditionReferenceSerializer
from dfhir.coverages.serializers import CoverageReferenceSerializer
from dfhir.devices.serializers import DeviceReferenceSerializer
from dfhir.encounters.serializers import EncounterReferenceSerializer
from dfhir.locations.serializers import (
    LocationOrganizationReferenceSerializer,
    LocationReferenceSerializer,
)
from dfhir.patients.serializers import PatientReferenceSerializer
from dfhir.procedures.serializers import ProcedureReferenceSerializer
from dfhir.servicerequests.serializers import ServiceRequestReferenceSerializer

from .models import (
    ExplanationOfBenefit,
    ExplanationOfBenefitAccident,
    ExplanationOfBenefitAddItem,
    ExplanationOfBenefitAddItemDetail,
    ExplanationOfBenefitBenefitBalance,
    ExplanationOfBenefitBenefitBalanceFinancial,
    ExplanationOfBenefitCareTeam,
    ExplanationOfBenefitDiagnosis,
    ExplanationOfBenefitEntererReference,
    ExplanationOfBenefitEvent,
    ExplanationOfBenefitInsurance,
    ExplanationOfBenefitItem,
    ExplanationOfBenefitItemAdjudication,
    ExplanationOfBenefitItemBodySite,
    ExplanationOfBenefitItemDetail,
    ExplanationOfBenefitItemDetailSubDetail,
    ExplanationOfBenefitItemRequestReference,
    ExplanationOfBenefitItemReviewOutcome,
    ExplanationOfBenefitPayee,
    ExplanationOfBenefitPayeePartyReference,
    ExplanationOfBenefitPayment,
    ExplanationOfBenefitPrescriptionReference,
    ExplanationOfBenefitProcedure,
    ExplanationOfBenefitProcessNote,
    ExplanationOfBenefitProviderReference,
    ExplanationOfBenefitRelated,
    ExplanationOfBenefitSupportingInfo,
    ExplanationOfBenefitTotal,
)


class ExplanationOfBenefitPrescriptionReferenceSerializer(BaseReferenceModelSerializer):
    """Explanation of Benefit Prescription Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitPrescriptionReference
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitProviderReferenceSerializer(BaseReferenceModelSerializer):
    """Explanation of Benefit Provider Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitProviderReference
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitItemRequestReferenceSerializer(BaseReferenceModelSerializer):
    """Explanation of Benefit Item Request Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitItemRequestReference
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitsRelatedSerializer(WritableNestedModelSerializer):
    """Explanation of Benefits Related Serializer."""

    claim = ClaimReferenceSerializer(many=False, required=False)
    relationship = CodeableConceptSerializer(many=False, required=False)
    reference = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitRelated
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitBenefitBalanceFinancialSerializer(
    WritableNestedModelSerializer
):
    """Explanation of Benefit Benefit Balance Financial Serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    allowed_money = MoneySerializer(many=False, required=False)
    used_money = MoneySerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitBenefitBalanceFinancial
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitItemBodySiteSerializer(WritableNestedModelSerializer):
    """Explanation of Benefit Item Body Site Serializer."""

    site = BodyStructureCodeableReferenceSerializer(many=True, required=False)
    sub_site = CodingSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitItemBodySite
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitItemReviewOutcomeSerializer(WritableNestedModelSerializer):
    """Explanation of Benefit Item Review Outcome Serializer."""

    code = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitItemReviewOutcome
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitItemAdjudicationSerializer(WritableNestedModelSerializer):
    """Explanation of Benefit Item Adjudication Serializer."""

    category = CodeableConceptSerializer(many=False, required=False)
    reason = CodeableConceptSerializer(many=False, required=False)
    amount = MoneySerializer(many=False, required=False)
    quantity = QuantitySerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitItemAdjudication
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitItemDetailSubDetailSerializer(WritableNestedModelSerializer):
    """Explanation of Benefit Item Detail Sub Detail Serializer."""

    trace_number = IdentifierSerializer(many=True, required=False)
    revenue = CodeableConceptSerializer(many=False, required=False)
    category = CodeableConceptSerializer(many=False, required=False)
    product_or_service = CodeableConceptSerializer(many=False, required=False)
    product_or_service_end = CodeableConceptSerializer(many=False, required=False)
    modifier = CodeableConceptSerializer(many=True, required=False)
    program_code = CodeableConceptSerializer(many=True, required=False)
    patient_paid = MoneySerializer(many=False, required=False)
    quantity = SimpleQuantitySerializer(many=False, required=False)
    unit_price = MoneySerializer(many=False, required=False)
    tax = MoneySerializer(many=False, required=False)
    net = MoneySerializer(many=False, required=False)
    udi = DeviceReferenceSerializer(many=True, required=False)
    review_outcome = ExplanationOfBenefitItemReviewOutcomeSerializer(
        many=False, required=False
    )
    adjudication = ExplanationOfBenefitItemAdjudicationSerializer(
        many=True, required=False
    )

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitItemDetailSubDetail
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitAddItemDetailSerializer(WritableNestedModelSerializer):
    """Explanation of Benefit Add Item Detail Serializer."""

    trace_number = IdentifierSerializer(many=True, required=False)
    revenue = CodeableConceptSerializer(many=False, required=False)
    product_or_service = CodeableConceptSerializer(many=False, required=False)
    product_or_service_end = CodeableConceptSerializer(many=False, required=False)
    modifier = CodeableConceptSerializer(many=True, required=False)
    patient_paid = MoneySerializer(many=False, required=False)
    quantity = SimpleQuantitySerializer(many=False, required=False)
    unit_price = MoneySerializer(many=False, required=False)
    tax = MoneySerializer(many=False, required=False)
    net = MoneySerializer(many=False, required=False)
    review_outcome = ExplanationOfBenefitItemReviewOutcomeSerializer(
        many=False, required=False
    )
    adjudication = ExplanationOfBenefitItemAdjudicationSerializer(
        many=True, required=False
    )
    sub_detail = ExplanationOfBenefitItemDetailSubDetailSerializer(
        many=True, required=False
    )

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitAddItemDetail
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitBenefitBalanceSerializer(WritableNestedModelSerializer):
    """Explanation of Benefit Benefit Balance Serializer."""

    category = CodeableConceptSerializer(many=False, required=False)
    network = CodeableConceptSerializer(many=False, required=False)
    unit = CodeableConceptSerializer(many=False, required=False)
    term = CodeableConceptSerializer(many=False, required=False)
    financial = ExplanationOfBenefitBenefitBalanceFinancialSerializer(
        many=True, required=False
    )

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitBenefitBalance
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitPayeePartyReferenceSerializer(BaseReferenceModelSerializer):
    """Explanation of Benefit Payee Party Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitPayeePartyReference
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitPayeeSerializer(WritableNestedModelSerializer):
    """Explanation of Benefit Payee Serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    party = ExplanationOfBenefitPayeePartyReferenceSerializer(
        many=False, required=False
    )

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitPayee
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitDiagnosisSerializer(WritableNestedModelSerializer):
    """Explanation of Benefit Diagnosis Serializer."""

    diagnosis_codeable_concept = CodeableConceptSerializer(many=False, required=False)
    diagnosis_reference = ConditionReferenceSerializer(many=False, required=False)
    type = CodeableConceptSerializer(many=True, required=False)
    on_admission = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitDiagnosis
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitItemDetailSerializer(WritableNestedModelSerializer):
    """Explanation of Benefit Item Detail Serializer."""

    trace_number = IdentifierSerializer(many=True, required=False)
    revenue = CodeableConceptSerializer(many=False, required=False)
    category = CodeableConceptSerializer(many=False, required=False)
    product_or_service = CodeableConceptSerializer(many=False, required=False)
    product_or_service_end = CodeableConceptSerializer(many=False, required=False)
    modifier = CodeableConceptSerializer(many=True, required=False)
    program_code = CodeableConceptSerializer(many=True, required=False)
    patient_paid = MoneySerializer(many=False, required=False)
    quantity = SimpleQuantitySerializer(many=False, required=False)
    unit_price = MoneySerializer(many=False, required=False)
    tax = MoneySerializer(many=False, required=False)
    net = MoneySerializer(many=False, required=False)
    udi = DeviceReferenceSerializer(many=True, required=False)
    review_outcome = ExplanationOfBenefitItemReviewOutcomeSerializer(
        many=False, required=False
    )
    adjudication = ExplanationOfBenefitItemAdjudicationSerializer(
        many=True, required=False
    )
    sub_detail = ExplanationOfBenefitItemDetailSubDetailSerializer(
        many=True, required=False
    )

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitItemDetail
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitItemSerializer(WritableNestedModelSerializer):
    """Explanation of Benefit Item Serializer."""

    trace_number = IdentifierSerializer(many=True, required=False)
    revenue = CodeableConceptSerializer(many=False, required=False)
    category = CodeableConceptSerializer(many=False, required=False)
    product_or_service = CodeableConceptSerializer(many=False, required=False)
    product_or_service_end = CodeableConceptSerializer(many=False, required=False)
    request = ExplanationOfBenefitItemRequestReferenceSerializer(
        many=True, required=False
    )
    modifier = CodeableConceptSerializer(many=True, required=False)
    program_code = CodeableConceptSerializer(many=True, required=False)
    serviced_period = PeriodSerializer(many=False, required=False)
    location_codeable_concept = CodeableConceptSerializer(many=False, required=False)
    location_address = AddressSerializer(many=False, required=False)
    location_reference = LocationReferenceSerializer(many=False, required=False)
    patient_paid = MoneySerializer(many=False, required=False)
    quantity = SimpleQuantitySerializer(many=False, required=False)
    unit_price = MoneySerializer(many=False, required=False)
    tax = MoneySerializer(many=False, required=False)
    net = MoneySerializer(many=False, required=False)
    udi = DeviceReferenceSerializer(many=True, required=False)
    body_site = ExplanationOfBenefitItemBodySiteSerializer(many=False, required=False)
    encounter = EncounterReferenceSerializer(many=True, required=False)
    review_outcome = ExplanationOfBenefitItemReviewOutcomeSerializer(
        many=False,
        required=False,
    )
    adjudication = ExplanationOfBenefitItemAdjudicationSerializer(
        many=True, required=False
    )
    detail = ExplanationOfBenefitItemDetailSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitItem
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitAccidentSerializer(WritableNestedModelSerializer):
    """Explanation of Benefit Accident Serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    location_address = AddressSerializer(many=False, required=False)
    location_reference = LocationReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitAccident
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitInsuranceSerializer(WritableNestedModelSerializer):
    """Explanation of Benefit Insurance Serializer."""

    focal = serializers.BooleanField(required=False)
    coverage = CoverageReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitInsurance
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitEventSerializer(WritableNestedModelSerializer):
    """Explanation of Benefit Event Serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    when_period = PeriodSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitEvent
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitPaymentSerializer(WritableNestedModelSerializer):
    """Explanation of Benefit Payment Serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    adjustment = MoneySerializer(many=False, required=False)
    adjustment_reason = CodeableConceptSerializer(many=False, required=False)
    amount = MoneySerializer(many=False, required=False)
    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitPayment
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitProcedureSerializer(WritableNestedModelSerializer):
    """Explanation of Benefit Procedure Serializer."""

    type = CodeableConceptSerializer(many=True, required=False)
    procedure_codeable_concept = CodeableConceptSerializer(many=False, required=False)
    procedure_reference = ProcedureReferenceSerializer(many=False, required=False)
    udi = DeviceReferenceSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitProcedure
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitTotalSerializer(WritableNestedModelSerializer):
    """Explanation of Benefit Total Serializer."""

    category = CodeableConceptSerializer(many=False, required=False)
    amount = MoneySerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitTotal
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitAddItemSerializer(WritableNestedModelSerializer):
    """Explanation of Benefit Add Item Serializer."""

    trace_number = IdentifierSerializer(many=True, required=False)
    provider = ExplanationOfBenefitProviderReferenceSerializer(
        many=True, required=False
    )
    revenue = CodeableConceptSerializer(many=False, required=False)
    product_or_Service = CodeableConceptSerializer(many=False, required=False)
    product_or_service_end = CodeableConceptSerializer(many=False, required=False)
    request = ExplanationOfBenefitItemRequestReferenceSerializer(
        many=True, required=False
    )
    modifier = CodeableConceptSerializer(many=True, required=False)
    program_code = CodeableConceptSerializer(many=True, required=False)
    serviced_period = PeriodSerializer(many=False, required=False)
    location_codeable_concept = CodeableConceptSerializer(many=False, required=False)
    location_address = AddressSerializer(many=False, required=False)
    location_reference = LocationReferenceSerializer(many=False, required=False)
    patient_paid = MoneySerializer(many=False, required=False)
    quantity = SimpleQuantitySerializer(many=False, required=False)
    unit_price = MoneySerializer(many=False, required=False)
    tax = MoneySerializer(many=False, required=False)
    net = MoneySerializer(many=False, required=False)
    body_site = ExplanationOfBenefitItemBodySiteSerializer(many=False, required=False)
    review_outcome = ExplanationOfBenefitItemReviewOutcomeSerializer(
        many=False, required=False
    )
    adjudication = ExplanationOfBenefitItemAdjudicationSerializer(
        many=True, required=False
    )
    detail = ExplanationOfBenefitAddItemDetailSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitAddItem
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitCareTeamSerializer(WritableNestedModelSerializer):
    """Explanation of Benefit Care Team Serializer."""

    provider = ExplanationOfBenefitProviderReferenceSerializer(
        many=False, required=False
    )
    role = CodeableConceptSerializer(many=False, required=False)
    specialty = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitCareTeam
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitEntererReferenceSerializer(BaseReferenceModelSerializer):
    """Explanation of Benefit Enterer Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitEntererReference
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitProcessNoteSerializer(WritableNestedModelSerializer):
    """Explanation of Benefit Process Note Serializer."""

    type = CodeableConceptSerializer(many=False, required=False)
    language = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitProcessNote
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitSupportingInfoSerializer(WritableNestedModelSerializer):
    """Explanation of Benefit Supporting Info Serializer."""

    category = CodeableConceptSerializer(many=False, required=False)
    code = CodeableConceptSerializer(many=False, required=False)
    timing_period = PeriodSerializer(many=False, required=False)
    value_quantity = QuantitySerializer(many=False, required=False)
    value_attachment = AttachmentSerializer(many=False, required=False)
    value_reference = ReferenceSerializer(many=False, required=False)
    value_identifier = IdentifierSerializer(many=False, required=False)
    reason = CodingSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefitSupportingInfo
        exclude = ["created_at", "updated_at"]


class ExplanationOfBenefitSerializer(BaseWritableNestedModelSerializer):
    """Explanation of Benefits Serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    trace_number = IdentifierSerializer(many=True, required=False)
    type = CodeableConceptSerializer(many=False, required=False)
    sub_type = CodeableConceptSerializer(many=False, required=False)
    patient = PatientReferenceSerializer(many=False, required=False)
    billable_period = PeriodSerializer(many=False, required=False)
    enterer = ExplanationOfBenefitEntererReferenceSerializer(many=False, required=False)
    insurer = OrganizationReferenceSerializer(many=False, required=False)
    provider = ExplanationOfBenefitProviderReferenceSerializer(
        many=False, required=False
    )
    priority = CodeableConceptSerializer(many=False, required=False)
    funds_reserve_requested = CodeableConceptSerializer(many=False, required=False)
    funds_reserve = CodeableConceptSerializer(many=False, required=False)
    related = ExplanationOfBenefitsRelatedSerializer(many=True, required=False)
    prescription = ExplanationOfBenefitPrescriptionReferenceSerializer(
        many=False, required=False
    )
    original_prescription = ExplanationOfBenefitPrescriptionReferenceSerializer(
        many=False, required=False
    )
    event = ExplanationOfBenefitEventSerializer(many=True, required=False)
    payee = ExplanationOfBenefitPayeeSerializer(many=False, required=False)
    referral = ServiceRequestReferenceSerializer(many=False, required=False)
    facility = LocationOrganizationReferenceSerializer(many=False, required=False)
    claim = ClaimReferenceSerializer(many=False, required=False)
    claim_response = ClaimResponseReferenceSerializer(required=False)
    decision = CodeableConceptSerializer(many=False, required=False)
    pre_auth_period = PeriodSerializer(many=False, required=False)
    diagnosis_related_group = CodeableConceptSerializer(many=False, required=False)
    care_team = ExplanationOfBenefitCareTeamSerializer(many=True, required=False)
    supporting_info = ExplanationOfBenefitSupportingInfoSerializer(
        many=True, required=False
    )
    diagnosis = ExplanationOfBenefitDiagnosisSerializer(many=True, required=False)
    procedure = ExplanationOfBenefitProcedureSerializer(many=True, required=False)
    insurance = ExplanationOfBenefitInsuranceSerializer(many=True, required=False)
    accident = ExplanationOfBenefitAccidentSerializer(many=False, required=False)
    patient_paid = MoneySerializer(many=False, required=False)
    item = ExplanationOfBenefitItemSerializer(many=True, required=False)
    add_item = ExplanationOfBenefitAddItemSerializer(many=True, required=False)
    adjudication = ExplanationOfBenefitItemAdjudicationSerializer(
        many=True, required=False
    )
    total = ExplanationOfBenefitTotalSerializer(many=True, required=False)
    payment = ExplanationOfBenefitPaymentSerializer(many=False, required=False)
    form_code = CodeableConceptSerializer(many=False, required=False)
    form = AttachmentSerializer(many=False, required=False)
    process_note = ExplanationOfBenefitProcessNoteSerializer(many=True, required=False)
    benefit_period = PeriodSerializer(many=False, required=False)
    benefit_balance = ExplanationOfBenefitBenefitBalanceSerializer(
        many=True, required=False
    )

    class Meta:
        """Meta class."""

        model = ExplanationOfBenefit
        exclude = ["created_at", "updated_at"]



"""Clinical impressions serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    PeriodSerializer,
    ReferenceSerializer,
)
from dfhir.conditions.serializers import ConditionAllergyIntoleranceReferenceSerializer
from dfhir.encounters.serializers import EncounterReferenceSerializer
from dfhir.patients.serializers import PatientGroupReferenceSerializer
from dfhir.practitioners.serializers import (
    PractitionerPractitionerRoleReferenceSerializer,
)

from .models import (
    ClinicalImpression,
    ClinicalImpressionFinding,
    ClinicalImpressionFindingItemCodeableReference,
    ClinicalImpressionFindingItemReference,
    ClinicalImpressionReference,
)


class ClinicalImpressionReferenceSerializer(BaseReferenceModelSerializer):
    """Clinical Impression Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = ClinicalImpressionReference
        exclude = ["created_at", "updated_at"]


class ClinicalImpressionFindingItemReferenceSerializer(BaseReferenceModelSerializer):
    """Clinical Impression Finding Item Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = ClinicalImpressionFindingItemReference
        exclude = ["created_at", "updated_at"]


class ClinicalImpressionFindingItemCodeableReferenceSerializer(
    WritableNestedModelSerializer
):
    """Clinical Impression Finding Item Codeable Reference Serializer."""

    reference = ClinicalImpressionFindingItemReferenceSerializer(
        many=False, required=False
    )
    concept = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ClinicalImpressionFindingItemCodeableReference
        exclude = ["created_at", "updated_at"]


class ClinicalImpressionFindingSerializer(BaseWritableNestedModelSerializer):
    """Clinical Impression Finding Serializer."""

    item = ClinicalImpressionFindingItemCodeableReferenceSerializer(
        many=False, required=False
    )

    class Meta:
        """Meta class."""

        model = ClinicalImpressionFinding
        exclude = ["created_at", "updated_at"]


class ClinicalImpressionSerializer(BaseWritableNestedModelSerializer):
    """Clinical Impression Serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    status_reason = CodeableConceptSerializer(many=False, required=False)
    subject = PatientGroupReferenceSerializer(many=False, required=False)
    encounter = EncounterReferenceSerializer(many=False, required=False)
    effective_period = PeriodSerializer(many=False, required=False)
    performer = PractitionerPractitionerRoleReferenceSerializer(
        many=False, required=False
    )
    previous = ClinicalImpressionReferenceSerializer(many=False, required=False)
    problem = ConditionAllergyIntoleranceReferenceSerializer(many=True, required=False)
    change_pattern = CodeableConceptSerializer(many=False, required=False)
    finding = ClinicalImpressionFindingSerializer(many=True, required=False)
    prognosis_codeable_concept = CodeableConceptSerializer(many=True, required=False)
    supporting_info = ReferenceSerializer(many=True, required=False)
    note = AnnotationSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = ClinicalImpression
        exclude = ["created_at", "updated_at"]



# Generated by Django 5.2 on 2025-05-05 14:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('allergyintolerances', '0006_allergyintolerancereference'),
        ('nutritionorders', '0003_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='nutritionorder',
            name='allergy_intolerance',
            field=models.ManyToManyField(blank=True, to='allergyintolerances.allergyintolerancereference'),
        ),
    ]



"""Contract views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import Contract
from .serializers import ContractSerializer


class ContractListView(APIView):
    """Contract list create view."""

    permission_classes = [AllowAny]

    @extend_schema(request=ContractSerializer, responses={201: ContractSerializer})
    def post(self, request):
        """Create a contract."""
        serializer = ContractSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)

    @extend_schema(responses={200: ContractSerializer(many=True)})
    def get(self, request):
        """Get all contracts."""
        contracts = Contract.objects.all()
        serializer = ContractSerializer(contracts, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)


class ContractDetailView(APIView):
    """Contract detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get contract object."""
        try:
            return Contract.objects.get(id=pk)
        except Contract.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: ContractSerializer})
    def get(self, request, pk=None):
        """Get contract detail."""
        contract = self.get_object(pk)
        serializer = ContractSerializer(contract)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(request=ContractSerializer, responses={200: ContractSerializer})
    def patch(self, request, pk=None):
        """Update contract."""
        contract = self.get_object(pk)
        serializer = ContractSerializer(contract, data=request.data, partial=True)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk=None):
        """Delete contract."""
        contract = self.get_object(pk)
        contract.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('careplans', '0003_initial'),
        ('encounters', '0002_initial'),
        ('locations', '0001_initial'),
        ('medicationdispenses', '0001_initial'),
        ('medicationrequests', '0001_initial'),
        ('medications', '0001_initial'),
        ('provenances', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='medicationdispense',
            name='authorizing_prescription',
            field=models.ManyToManyField(blank=True, related_name='medication_dispense_authorizing_prescription', to='medicationrequests.medicationrequestreference'),
        ),
        migrations.AddField(
            model_name='medicationdispense',
            name='based_on',
            field=models.ManyToManyField(blank=True, to='careplans.careplanreference'),
        ),
        migrations.AddField(
            model_name='medicationdispense',
            name='category',
            field=models.ManyToManyField(blank=True, related_name='medication_dispense_category', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='medicationdispense',
            name='days_supply',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_dispense_days_supply', to='base.quantity'),
        ),
        migrations.AddField(
            model_name='medicationdispense',
            name='destination',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_dispense_destination', to='locations.locationreference'),
        ),
        migrations.AddField(
            model_name='medicationdispense',
            name='encounter',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_dispense_encounter', to='encounters.encounterreference'),
        ),
        migrations.AddField(
            model_name='medicationdispense',
            name='event_history',
            field=models.ManyToManyField(blank=True, related_name='medication_dispense_event_history', to='provenances.provenance'),
        ),
        migrations.AddField(
            model_name='medicationdispense',
            name='identifier',
            field=models.ManyToManyField(blank=True, related_name='medication_dispense_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='medicationdispense',
            name='location',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='medication_dispense_location', to='locations.locationreference'),
        ),
        migrations.AddField(
            model_name='medicationdispense',
            name='medication',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='medication_dispense_medication', to='medications.medicationcodeablereference'),
        ),
        migrations.AddField(
            model_name='medicationdispense',
            name='note',
            field=models.ManyToManyField(blank=True, related_name='medication_dispense_note', to='base.annotation'),
        ),
    ]



"""EnrollmentRequests URL Configuration."""

from django.urls import path

from . import views

app_name = "enrollmentrequests"

urlpatterns = [
    path(
        "enrollmentrequests/",
        views.EnrollmentRequestListView.as_view(),
        name="list-create",
    ),
    path(
        "enrollmentrequests/<int:pk>/",
        views.EnrollmentRequestDetailView.as_view(),
        name="detail",
    ),
]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.contrib.postgres.fields
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('documentreferences', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='MolecularDefinitionCytobandIntervalStartEndCytoband',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('arm_code', models.CharField(max_length=255, null=True)),
                ('arm_string', models.CharField(max_length=255, null=True)),
                ('region_code', models.CharField(max_length=255, null=True)),
                ('region_string', models.CharField(max_length=255, null=True)),
                ('band_code', models.CharField(max_length=255, null=True)),
                ('band_string', models.CharField(max_length=255, null=True)),
                ('sub_band_code', models.CharField(max_length=255, null=True)),
                ('sub_band_string', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MolecularDefinitionRepresentationConcatenated',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MolecularDefinitionCoordinateIntervalCoordinateSystem',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('end_quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='molecular_definition_coordinate_interval_coordinate_system_end_quantity', to='base.quantity')),
                ('end_range', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='molecular_definition_coordinate_interval_coordinate_system_end_range', to='base.range')),
                ('normalization_method', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='molecular_definition_coordinate_interval_coordinate_system_method', to='base.codeableconcept')),
                ('origin', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='molecular_definition_coordinate_interval_coordinate_system_origin', to='base.codeableconcept')),
                ('start_quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='molecular_definition_coordinate_interval_coordinate_system_start_quantity', to='base.quantity')),
                ('start_range', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='molecular_definition_coordinate_interval_coordinate_system_start_range', to='base.range')),
                ('system', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='molecular_definition_coordinate_interval_coordinate_system_system', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MolecularDefinitionCoordinateInterval',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('coordinate_system', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='molecular_definition_relative_edit_coordinate_system', to='moleculardefinitions.moleculardefinitioncoordinateintervalcoordinatesystem')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MolecularDefinitionCytobandInterval',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('chromosome', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='cytoband_interval_chromosome', to='base.codeableconcept')),
                ('end_cytoband', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='cytoband_interval_end_cytoband', to='moleculardefinitions.moleculardefinitioncytobandintervalstartendcytoband')),
                ('start_cytoband', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='cytoband_interval_start_cytoband', to='moleculardefinitions.moleculardefinitioncytobandintervalstartendcytoband')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MolecularDefinitionCytobandLocation',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('cytoband_interval', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='cytoband_location_cytoband_interval', to='moleculardefinitions.moleculardefinitioncytobandinterval')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MolecularDefinitionLocation',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('featured_location', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=255, null=True), null=True, size=None)),
                ('cytoband_location', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='molecular_definition_cytoband_location', to='moleculardefinitions.moleculardefinitioncytobandlocation')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MolecularDefinition',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('description', models.TextField(null=True)),
                ('identifier', models.ManyToManyField(blank=True, related_name='molecular_definition_identifier', to='base.identifier')),
                ('molecular_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='molecular_definition_molecular_type', to='base.codeableconcept')),
                ('topology', models.ManyToManyField(blank=True, related_name='molecular_definition_topology', to='base.codeableconcept')),
                ('type', models.ManyToManyField(blank=True, related_name='molecular_definition_type', to='base.codeableconcept')),
                ('location', models.ManyToManyField(blank=True, related_name='molecular_definition_location', to='moleculardefinitions.moleculardefinitionlocation')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='MolecularDefinitionReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='molecular_definition_reference_identifier', to='base.identifier')),
                ('molecular_definition', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='molecular_definition_reference_molecular_definition', to='moleculardefinitions.moleculardefinition')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='moleculardefinition',
            name='member_state',
            field=models.ManyToManyField(blank=True, related_name='molecular_definition_member_state', to='moleculardefinitions.moleculardefinitionreference'),
        ),
        migrations.CreateModel(
            name='MolecularDefinitionRepresentation',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('code', models.ManyToManyField(blank=True, related_name='representation_code', to='base.codeableconcept')),
                ('focus', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='representation_focus', to='base.codeableconcept')),
                ('resolvable', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='representation_resolvable', to='documentreferences.documentreferencereference')),
                ('concatenated', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='representation_concatenated', to='moleculardefinitions.moleculardefinitionrepresentationconcatenated')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='moleculardefinition',
            name='representation',
            field=models.ManyToManyField(blank=True, related_name='molecular_definition_representation', to='moleculardefinitions.moleculardefinitionrepresentation'),
        ),
        migrations.CreateModel(
            name='MolecularDefinitionRepresentationConcatenatedSequenceElement',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('ordinal_index', models.IntegerField(null=True)),
                ('sequence', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='representation_concatenated_sequence_element_sequence', to='moleculardefinitions.moleculardefinitionreference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='moleculardefinitionrepresentationconcatenated',
            name='sequence_element',
            field=models.ManyToManyField(blank=True, related_name='representation_concatenated', to='moleculardefinitions.moleculardefinitionrepresentationconcatenatedsequenceelement'),
        ),
        migrations.CreateModel(
            name='MolecularDefinitionRepresentationExtracted',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reverse_complement', models.BooleanField(default=False)),
                ('coordinate_interval', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='molecular_definition_extracted_coordinate_interval', to='moleculardefinitions.moleculardefinitioncoordinateinterval')),
                ('starting_molecule', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='representation_extract_string_molecule', to='moleculardefinitions.moleculardefinitionreference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='moleculardefinitionrepresentation',
            name='extracted',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='representation_extracted', to='moleculardefinitions.moleculardefinitionrepresentationextracted'),
        ),
        migrations.CreateModel(
            name='MolecularDefinitionRepresentationLiteral',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('value', models.CharField(max_length=255, null=True)),
                ('encoding', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='representation_literal_encoding', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='moleculardefinitionrepresentation',
            name='literal',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='representation_literal', to='moleculardefinitions.moleculardefinitionrepresentationliteral'),
        ),
        migrations.CreateModel(
            name='MolecularDefinitionRepresentationRelative',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('starting_molecule', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='representation_starting_molecule', to='moleculardefinitions.moleculardefinitionreference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='moleculardefinitionrepresentation',
            name='relative',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='representation_relative', to='moleculardefinitions.moleculardefinitionrepresentationrelative'),
        ),
        migrations.CreateModel(
            name='MolecularDefinitionRepresentationRelativeEdit',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('edit_order', models.IntegerField(null=True)),
                ('coordinate_interval', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='molecular_representation_relative_edit_coordinate_interval', to='moleculardefinitions.moleculardefinitioncoordinateinterval')),
                ('coordinate_system', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='molecular_representation_coordinate_system', to='base.codeableconcept')),
                ('replaced_molecule', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='molecular_representation_edit_replaced_molecule', to='base.codeableconcept')),
                ('replacement_molecule', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='molecular_representation_relative_edit_replacement_molecule', to='moleculardefinitions.moleculardefinitionreference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='moleculardefinitionrepresentationrelative',
            name='edit',
            field=models.ManyToManyField(blank=True, related_name='representation_edit_relative', to='moleculardefinitions.moleculardefinitionrepresentationrelativeedit'),
        ),
        migrations.CreateModel(
            name='MolecularDefinitionRepresentationRepeated',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('copy_count', models.IntegerField(null=True)),
                ('sequence_motif', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='representation_extract_sequence_motif', to='moleculardefinitions.moleculardefinitionreference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='moleculardefinitionrepresentation',
            name='repeated',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='representation_repeat', to='moleculardefinitions.moleculardefinitionrepresentationrepeated'),
        ),
        migrations.CreateModel(
            name='MolecularDefinitionSequenceLocation',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('coordinated_interval', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='sequence_location_coordinated_internal', to='moleculardefinitions.moleculardefinitioncoordinateinterval')),
                ('sequence_context', models.ForeignKey(blank=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='sequence_location_sequence_context', to='moleculardefinitions.moleculardefinitionreference')),
                ('strand', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='sequence_location_strand', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='moleculardefinitionlocation',
            name='sequence_location',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='molecular_definition_sequence_location', to='moleculardefinitions.moleculardefinitionsequencelocation'),
        ),
        migrations.CreateModel(
            name='MolecularSequenceGenomeAssembly',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('description_markdown', models.TextField(null=True)),
                ('description_string', models.CharField(max_length=255, null=True)),
                ('accession', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genome_assembly_accession', to='base.codeableconcept')),
                ('build', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genome_assembly_build', to='base.codeableconcept')),
                ('organism', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genome_assembly_organism', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='moleculardefinitioncytobandlocation',
            name='genome_assembly',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='cytoband_location_genome_assembly', to='moleculardefinitions.molecularsequencegenomeassembly'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0004_initial'),
        ('devices', '0001_initial'),
        ('invoices', '0001_initial'),
        ('patients', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='invoice',
            name='subject',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_subject', to='patients.patientgroupreference'),
        ),
        migrations.AddField(
            model_name='invoice',
            name='total_gross',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_total_gross', to='base.money'),
        ),
        migrations.AddField(
            model_name='invoice',
            name='total_net',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_total_net', to='base.money'),
        ),
        migrations.AddField(
            model_name='invoice',
            name='total_price_component',
            field=models.ManyToManyField(blank=True, related_name='invoice_total_price_component', to='base.monetarycomponent'),
        ),
        migrations.AddField(
            model_name='invoice',
            name='type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='invoicelineitem',
            name='charge_item_codeable_concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_line_item_charge_item_codeable_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='invoicelineitem',
            name='price_component',
            field=models.ManyToManyField(blank=True, related_name='invoice_line_item_price_component', to='base.monetarycomponent'),
        ),
        migrations.AddField(
            model_name='invoicelineitem',
            name='service_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_line_item_service_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='invoice',
            name='line_item',
            field=models.ManyToManyField(blank=True, related_name='invoice_line_item', to='invoices.invoicelineitem'),
        ),
        migrations.AddField(
            model_name='invoiceparticipant',
            name='role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_participant_role', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='invoice',
            name='participant',
            field=models.ManyToManyField(blank=True, related_name='invoice_participant', to='invoices.invoiceparticipant'),
        ),
        migrations.AddField(
            model_name='invoiceparticipantactorreference',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_participant_actor_reference_device', to='devices.device'),
        ),
        migrations.AddField(
            model_name='invoiceparticipantactorreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_participant_actor_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='invoiceparticipantactorreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_participant_actor_reference_organization', to='base.organizationreference'),
        ),
        migrations.AddField(
            model_name='invoiceparticipantactorreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='invoice_participant_actor_reference_patient', to='patients.patient'),
        ),
    ]



"""Healthcare Services Choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class EligibilityCode(models.TextChoices):
    """Eligibility code choices."""

    VETERANS = "veterans", _("Veterans")
    NEW_PATIENT = "new_patient", _("New Patient")
    PADIATIC_PATIENT = "padiatic_patient", _("Padiatic Patient")
    LOW_INCOME_PATEINT = "low_income_patient", _("Low Income Patient")
    UNINSURED_PATIENT = "uninsure_patient", _("Uninsured Patient")
    RENAL_PATIENT = "renal_patient", _("Renal Patient")
    SPECIALIT_REFERAL_REQUIRED = (
        "specialist_referral_required",
        _("Specialist Referral Required"),
    )
    ASSESSMENT_REQUIRED = "assessment_required", _("Assessment Required")


class ServiceProvisionCode(models.TextChoices):
    """Service provision code choices."""

    FREE = "free", _("Free")
    DISCOUNTED = "discounted", _("Discounted")
    FEE_APPLY = "fee_apply", _("Fee Apply")


class ServiceModeChoices(models.TextChoices):
    """Service mode choices."""

    TELEPHONE = "telephone", _("Telephone")
    IN_PERSON = "in_person", _("In Person")
    VIDEO_CONFERENCE = "video_conference", _("Video Conference")
    CHAT = "chat", _("Chat")


class ReferralMethodChoices(models.TextChoices):
    """Referral method choices."""

    FAX = "fax", _("Fax")
    SECURE_EMAIL = "secure_email", _("Secure Email")
    PHONE = "phone", _("Phone")
    URL = "url", _("URL")
    SECURE_MESSAGING = "secure_messaging", _("Secure Messaging")
    MAIL = "mail", _("Mail")
    SELF_REFERRAL = "self_referral", _("Self Referral")



"""Flag views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import Flag
from .serializers import FlagSerializer


class FlagListView(APIView):
    """Flag list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: FlagSerializer(many=True)})
    def get(self, request):
        """Get all flags."""
        flags = Flag.objects.all()
        serializer = FlagSerializer(flags, many=True)
        return Response(serializer.data)

    @extend_schema(request=FlagSerializer, responses={201: FlagSerializer})
    def post(self, request):
        """Create a flag."""
        serializer = FlagSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class FlagDetailView(APIView):
    """Flag detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get flag object."""
        try:
            return Flag.objects.get(pk=pk)
        except Flag.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: FlagSerializer})
    def get(self, request, pk):
        """Get a flag."""
        flag = self.get_object(pk)
        serializer = FlagSerializer(flag)
        return Response(serializer.data)

    @extend_schema(request=FlagSerializer, responses={200: FlagSerializer})
    def put(self, request, pk):
        """Update a flag."""
        flag = self.get_object(pk)
        serializer = FlagSerializer(flag, data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data)

    @extend_schema(responses={204: None})
    def delete(self, request, pk):
        """Delete a flag."""
        flag = self.get_object(pk)
        flag.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('enrollmentresponses', '0001_initial'),
        ('practitionerroles', '0001_initial'),
        ('practitioners', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='enrollmentresponserequestproviderreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='enrollment_response_request_provider_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='enrollmentresponserequestproviderreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='enrollment_response_request_provider_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='enrollmentresponse',
            name='request_provider',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='enrollment_response_request_provider', to='enrollmentresponses.enrollmentresponserequestproviderreference'),
        ),
    ]



"""biologically derived product dispenses models."""

from django.db import models

from dfhir.base.models import BaseReference, Identifier, TimeStampedModel
from dfhir.biologicallyderivedproductdispenses.choices import (
    BiologicallyDerivedProductDispenseStatus,
)


class BiologicallyDerivedProductDispenseReference(BaseReference):
    """biologically derived product dispenses reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="biologicallyderivedproductdispense_reference_identifier",
    )
    biologically_derived_product_dispense = models.ForeignKey(
        "BiologicallyDerivedProductDispense",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="biologicallyderivedproductdispense_reference_biologicallyderivedproductdispense",
    )


class BiologicallyDerivedProductDispensePerformer(TimeStampedModel):
    """biologically derived product dispenses performer model."""

    function = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="biologicallyderivedproductdispense_performer_function",
    )
    actor = models.ForeignKey(
        "practitioners.PractitionerReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="biologicallyderivedproductdispense_performer_actor",
    )


class BiologicallyDerivedProductDispense(TimeStampedModel):
    """Biologicallyderivedproductdispense models."""

    identifier = models.ManyToManyField(
        Identifier,
        related_name="biologicallyderivedproductdispense_identifier",
        blank=True,
    )
    based_on = models.ManyToManyField(
        "servicerequests.ServiceRequestReference",
        blank=True,
        related_name="biologicallyderivedproductdispense_based_on",
    )
    part_of = models.ManyToManyField(
        BiologicallyDerivedProductDispenseReference,
        blank=True,
        related_name="biologicallyderivedproductdispense_part_of",
    )
    status = models.CharField(
        max_length=255,
        null=True,
        choices=BiologicallyDerivedProductDispenseStatus.choices,
    )
    original_relationship_type = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="biologicallyderivedproductdispense_original_relationship_type",
    )
    product = models.ForeignKey(
        BiologicallyDerivedProductDispenseReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="biologicallyderivedproductdispense_product",
    )
    patient = models.ForeignKey(
        "patients.PatientReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="biologicallyderivedproductdispense_patient",
    )
    match_status = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="biologicallyderivedproductdispense_match_status",
    )
    performer = models.ManyToManyField(
        BiologicallyDerivedProductDispensePerformer,
        blank=True,
        related_name="biologicallyderivedproductdispense_performer",
    )
    location = models.ForeignKey(
        "locations.LocationReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="biologicallyderivedproductdispense_location",
    )
    quantity = models.ForeignKey(
        "base.SimpleQuantity",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="biologicallyderivedproductdispense_quantity",
    )
    prepared_date = models.DateTimeField(null=True)
    when_handed_over = models.DateTimeField(null=True)
    destination = models.ForeignKey(
        "locations.LocationReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="biologicallyderivedproductdispense_destination",
    )
    note = models.ManyToManyField(
        "base.Annotation",
        blank=True,
        related_name="biologicallyderivedproductdispense_note",
    )
    usage_instruction = models.CharField(max_length=255, null=True)



# Generated by Django 5.2.1 on 2025-07-21 00:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('slots', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='slot',
            name='status',
            field=models.CharField(choices=[('free', 'Free'), ('busy', 'Busy'), ('busy-unavailable', 'Busy Unavailable'), ('busy-tentative', 'Busy Tentative'), ('entered-in-error', 'Entered in Error')], default='free', max_length=255),
        ),
    ]



"""Schedule URL Configuration."""

from django.urls import path

from . import views

app_name = "schedules"

urlpatterns = [
    path("schedules/", views.ScheduleListView.as_view(), name="schedule-list"),
    path(
        "schedules/<int:pk>/",
        views.ScheduleDetailView.as_view(),
        name="schedule-detail",
    ),
]



"""Practitioner-related choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class ServiceModeChoices(models.TextChoices):
    """Service mode choices."""

    IN_PERSON = "in_person", _("in-person")
    TELEPHONE = "telephone", _("telephone")
    VIDEO_CONFERENCE = "video_conference", _("video-conference")
    CHAT = (
        "chat",
        _("chat"),
    )



"""ClaimResponses views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import ClaimResponse
from .serializers import ClaimResponseSerializer


class ClaimResponseListView(APIView):
    """ClaimResponse list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: ClaimResponseSerializer(many=True)})
    def get(self, request):
        """Get a list of claim responses."""
        claim_responses = ClaimResponse.objects.all()
        serializer = ClaimResponseSerializer(claim_responses, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=ClaimResponseSerializer, responses={201: ClaimResponseSerializer}
    )
    def post(self, request):
        """Create a claim response."""
        serializer = ClaimResponseSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class ClaimResponseDetailView(APIView):
    """ClaimResponse detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get a claim response object."""
        try:
            return ClaimResponse.objects.get(pk=pk)
        except ClaimResponse.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: ClaimResponseSerializer})
    def get(self, request, pk=None):
        """Get a claim response."""
        claim_response = self.get_object(pk)
        serializer = ClaimResponseSerializer(claim_response)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=ClaimResponseSerializer, responses={200: ClaimResponseSerializer}
    )
    def patch(self, request, pk=None):
        """Update a claim response."""
        claim_response = self.get_object(pk)
        serializer = ClaimResponseSerializer(
            claim_response, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk=None):
        """Delete a claim response."""
        claim_response = self.get_object(pk)
        claim_response.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""Endpoint serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AvailabilitySerializer,
    BaseReferenceModelSerializer,
    CodeableConceptSerializer,
    ContactPointSerializer,
    IdentifierSerializer,
    OrganizationReferenceSerializer,
    PeriodSerializer,
)

from .models import Endpoint, EndpointPayload, EndpointReference


class EndpointPayloadSerializer(WritableNestedModelSerializer):
    """Endpoint Payload serializer."""

    type = CodeableConceptSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = EndpointPayload
        exclude = ["created_at", "updated_at"]


class EndpointSerializer(WritableNestedModelSerializer):
    """Endpoint serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    payload = EndpointPayloadSerializer(many=True, required=False)
    contact = ContactPointSerializer(many=True, required=False)
    connection_type = CodeableConceptSerializer(many=True, required=False)
    environmental_type = CodeableConceptSerializer(many=True, required=False)
    managing_organization = OrganizationReferenceSerializer(many=False, required=False)
    availability = AvailabilitySerializer(many=False, required=False)
    period = PeriodSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = Endpoint
        exclude = ["created_at", "updated_at"]


class EndpointReferenceSerializer(BaseReferenceModelSerializer):
    """EndpointReference serializer."""

    class Meta:
        """Meta class."""

        model = EndpointReference
        exclude = ["created_at", "updated_at", "endpoint"]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('appointments', '0003_initial'),
        ('base', '0005_initial'),
        ('careteams', '0004_initial'),
        ('communications', '0003_initial'),
        ('devices', '0001_initial'),
        ('documentreferences', '0004_initial'),
        ('groups', '0002_initial'),
        ('healthcareservices', '0001_initial'),
        ('locations', '0001_initial'),
        ('observations', '0002_initial'),
        ('patients', '0002_initial'),
        ('practitioners', '0001_initial'),
        ('servicerequests', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='appointmentbasedonreference',
            name='service_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='appointment_based_on_reference_service_request', to='servicerequests.servicerequest'),
        ),
        migrations.AddField(
            model_name='appointment',
            name='based_on',
            field=models.ManyToManyField(blank=True, related_name='appointment_based_on', to='appointments.appointmentbasedonreference'),
        ),
        migrations.AddField(
            model_name='appointmentparticipant',
            name='period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='appointment_participant_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='appointmentparticipant',
            name='type',
            field=models.ManyToManyField(blank=True, related_name='appointment_participant_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='appointment',
            name='participant',
            field=models.ManyToManyField(blank=True, related_name='appointment_participant', to='appointments.appointmentparticipant'),
        ),
        migrations.AddField(
            model_name='appointmentparticipantactor',
            name='care_team',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='appointment_participant_actor_care_team', to='careteams.careteam'),
        ),
        migrations.AddField(
            model_name='appointmentparticipantactor',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='appointment_participant_actor_device', to='devices.device'),
        ),
        migrations.AddField(
            model_name='appointmentparticipantactor',
            name='group',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='appointment_participant_actor_group', to='groups.group'),
        ),
        migrations.AddField(
            model_name='appointmentparticipantactor',
            name='healthcareservice',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='appointment_participant_actor_healthcareservice', to='healthcareservices.healthcareservice'),
        ),
        migrations.AddField(
            model_name='appointmentparticipantactor',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='appointment_participant_actor_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='appointmentparticipantactor',
            name='location',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='appointment_participant_actor_location', to='locations.location'),
        ),
        migrations.AddField(
            model_name='appointmentparticipantactor',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='appointment_participant_actor_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='appointmentparticipantactor',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='appointment_participant_actor_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='appointmentparticipant',
            name='actor',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='appointment_participant_actor', to='appointments.appointmentparticipantactor'),
        ),
        migrations.AddField(
            model_name='appointmentreasoncodeablereference',
            name='concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='appointment_reason_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='appointment',
            name='reason',
            field=models.ManyToManyField(blank=True, related_name='appointment_reason', to='appointments.appointmentreasoncodeablereference'),
        ),
        migrations.AddField(
            model_name='appointmentreasonreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='appointment_reason_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='appointmentreasonreference',
            name='observation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='appointment_reason_observation', to='observations.observation'),
        ),
        migrations.AddField(
            model_name='appointmentreasoncodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='appointment_reason_reference', to='appointments.appointmentreasonreference'),
        ),
        migrations.AddField(
            model_name='appointmentreference',
            name='appointment',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='appointment_reference_appointment', to='appointments.appointment'),
        ),
        migrations.AddField(
            model_name='appointmentreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='appointment_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='appointment',
            name='originating_appointment',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='appointment_originating_appointment', to='appointments.appointmentreference'),
        ),
        migrations.AddField(
            model_name='appointment',
            name='previous_appointment',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='appointment_previous_appointment', to='appointments.appointmentreference'),
        ),
        migrations.AddField(
            model_name='appointment',
            name='replaces',
            field=models.ManyToManyField(blank=True, related_name='appointment_replaces', to='appointments.appointmentreference'),
        ),
        migrations.AddField(
            model_name='documentreferencebinarycommunicationreference',
            name='communication',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_binary_communication_reference_communication', to='communications.communication'),
        ),
        migrations.AddField(
            model_name='documentreferencebinarycommunicationreference',
            name='document_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_binary_communication_reference_document_reference', to='documentreferences.documentreference'),
        ),
        migrations.AddField(
            model_name='documentreferencebinarycommunicationreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_binary_communication_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='appointment',
            name='patient_instruction',
            field=models.ManyToManyField(blank=True, related_name='appointment_patient_instruction', to='appointments.documentreferencebinarycommunicationreference'),
        ),
        migrations.AddField(
            model_name='monthlytemplate',
            name='day_of_week',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='monthly_template_day_of_week', to='base.coding'),
        ),
        migrations.AddField(
            model_name='monthlytemplate',
            name='nth_week_of_month',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='monthly_template_nth_week_of_month', to='base.coding'),
        ),
        migrations.AddField(
            model_name='recurrencetemplate',
            name='monthly_template',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='appointments.monthlytemplate'),
        ),
        migrations.AddField(
            model_name='recurrencetemplate',
            name='recurrence_type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='recurrence_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='recurrencetemplate',
            name='time_zone',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='recurrence_time_zone', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='recurrencetemplate',
            name='weekly_template',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='appointments.weeklytemplate'),
        ),
        migrations.AddField(
            model_name='recurrencetemplate',
            name='yearly_template',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='appointments.yearlytemplate'),
        ),
    ]



"""document reference urls."""

from django.urls import path

from dfhir.documentreferences.views import (
    DocumentReferenceDetailView,
    DocumentReferenceListView,
)

app_name = "documentreferences"

urlpatterns = [
    path(
        "documentreferences/",
        DocumentReferenceListView.as_view(),
        name="documentreference-list",
    ),
    path(
        "documentreferences/<int:pk>/",
        DocumentReferenceDetailView.as_view(),
        name="documentreference-detail",
    ),
]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0002_initial'),
        ('bodystructures', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ConditionAllergyIntoleranceReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ConditionAsserterReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ConditionCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ConditionObservationCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ConditionObservationReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ConditionRecorderReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ConditionReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ConditionStage',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ConditionStageAssessmentReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Condition',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('onset_date_time', models.DateTimeField(null=True)),
                ('onset_string', models.CharField(max_length=255, null=True)),
                ('abatement_date_time', models.DateTimeField(null=True)),
                ('abatement_string', models.CharField(max_length=255, null=True)),
                ('recorded_date', models.DateTimeField(null=True)),
                ('abatement_age', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='condition_abatement_age', to='base.age')),
                ('abatement_period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='condition_abatement_period', to='base.period')),
                ('abatement_range', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='condition_abatement_range', to='base.range')),
                ('body_site', models.ManyToManyField(blank=True, related_name='condition_body_site', to='base.codeableconcept')),
                ('body_structure', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='condition_body_structure', to='bodystructures.bodystructure')),
                ('category', models.ManyToManyField(blank=True, related_name='condition_category', to='base.codeableconcept')),
                ('clinical_status', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='condition_clinical_status', to='base.codeableconcept')),
                ('code', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='condition_code', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""Encounter model choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class EncounterStatus(models.TextChoices):
    """Choices for the status of an encounter."""

    PLANNED = "planned", _("Planned")
    IN_PROGRESS = "in-progress", _("In Progress")
    ON_HOLD = "on-hold", _("On Hold")
    DISCHARGED = "discharged", _("Discharged")
    COMPLETED = "completed", _("Completed")
    CANCELLED = "cancelled", _("Cancelled")
    DISCONTINUED = "discontinued", _("Discontinued")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")
    UNKNOWN = "unknown", _("Unknown")


class SubjectStatus(models.TextChoices):
    """Choices for the status of a subject."""

    ARRIVED = "arrived", _("Arrived")
    TRIAGED = "triaged", _("Triaged")
    RECEIVING_CARE = "receiving_care", _("Receiving Care")
    DEPARTED = "departed", _("Departed")
    ON_LEAVE = "on_leave", _("On Leave")


class AdmissionChoices(models.TextChoices):
    """Choices for admission types."""

    TRANSFERRED = "transferred", _("Transferred")
    OUTPATIENT_DEPARTMENT = "outpatient-department", _("Outpatient Department")
    EMERGENCY_DEPARTMENT = "emergency-department", _("Emergency or Accident Department")
    GP_REFERRAL = "gp-referral", _("General Practitioner Referral")
    MP_REFERRAL = "mp-referral", _("Medical Practitioner Referral")
    REHABILITATION = "rehabilitation", _("Rehabilitation Department")
    PSYCHIATRY = "psychiatry", _("Psychiatric Department")
    BORN_IN_HOSPITAL = "born-in-hospital", _("Born in Hospital")
    NURSING_HOME = "nursing-home", _("From Nursing Home")
    OTHER = "other", _("Other")


class DischargeChoices(models.TextChoices):
    """Choices for discharge destinations."""

    HOME = "home", _("Home")
    OTHER_HEALTH_CARE_FACILITY = (
        "other_health_care_facility",
        _("Other Health Care Facility"),
    )
    HOSPICE = "hospice", _("Hospice")
    LONG_TERM_CARE = "long-term-care", _("Long Term Care")
    LEFT_AGAINST_ADVICE = "left-against-advice", _("Left Against Advice")
    EXPIRED = "expired", _("Expired")
    PSYCHIATRIC_HOSPITAL = "psychiatric-hospital", _("Psychiatric Hospital")
    REHABILITATION = "rehabilitation", _("Rehabilitation")
    SKILLED_NURSING = "skilled-nursing", _("Skilled Nursing")
    ALTERNATIVE_HOME = "alternative-home", _("Alternative Home")
    OTHER = "other", _("Other")


class EncounterClassChoices(models.TextChoices):
    """Choices for the class of an encounter."""

    INPATIENT_ENCOUNTER = "inpatient-encounter", _("Inpatient Encounter")
    AMBULATORY = "outpatient-encounter", _("Outpatient Encounter")
    OBSERVATION_ENCOUNTER = "observation-encounter", _("Observation Encounter")
    EMERGENCY = "emergency", _("Emergency")
    VIRTUAL = "virtual", _("Virtual")
    HOME_HEALTH = "home-health", _("Home Health")


class EncounterPriority(models.TextChoices):
    """Choices for the priority of an encounter."""

    ASAP = "asap", _("ASAP")
    CALLBACK_RESULTS = "callback-results", _("Callback Results")
    ELECTIVE = "elective", _("Elective")
    EMERGENCY = "emergency", _("Emergency")
    PREOP = "preop", _("Preop")
    AS_NEEDED = "as-needed", _("As Needed")
    ROUTINE = "routine", _("Routine")
    RUSH_REPORTING = "rush-reporting", _("Rush Reporting")
    STAT = "stat", _("Stat")
    TIMING_CRITICAL = "timing-critical", _("Timing Critical")
    USE_AS_DIRECTED = "use-as-directed", _("Use As Directed")
    URGENT = "urgent", _("Urgent")
    CALLBACK_FOR_SCHEDULING = "callback-for-scheduling", _("Callback for Scheduling")
    CALLBACK_PLACER_FOR_SCHEDULING = (
        "callback-placer-for-scheduling",
        _("Callback Placer for Scheduling"),
    )
    CONTACT_PATIENT_FOR_SCHEDULING = (
        "contact-patient-for-scheduling",
        _("Contact Patient for Scheduling"),
    )


class EncounterLocationStatusChoices(models.TextChoices):
    """Choices for the status of an encounter location."""

    PLANNED = "planned", _("Planned")
    ACTIVE = "active", _("Active")
    RESERVED = "reserved", _("Reserved")
    COMPLETED = "completed", _("Completed")



"""biologically derived product dispenses serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    SimpleQuantitySerializer,
)
from dfhir.biologicallyderivedproductdispenses.models import (
    BiologicallyDerivedProductDispense,
    BiologicallyDerivedProductDispensePerformer,
    BiologicallyDerivedProductDispenseReference,
)
from dfhir.locations.serializers import LocationReferenceSerializer
from dfhir.patients.serializers import PatientReferenceSerializer
from dfhir.practitioners.serializers import PractitionerReferenceSerializer
from dfhir.servicerequests.serializers import ServiceRequestReferenceSerializer


class BiologicallyDerivedProductDispenseReferenceSerializer(
    BaseReferenceModelSerializer
):
    """biologically derived product dispense reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """meta options."""

        model = BiologicallyDerivedProductDispenseReference
        exclude = ["created_at", "updated_at"]


class BiologicallyDerivedProductDispensePerformerSerializer(
    WritableNestedModelSerializer
):
    """biologically derived product dispense performer serializer."""

    function = CodeableConceptSerializer(required=False)
    actor = PractitionerReferenceSerializer(required=False)

    class Meta:
        """meta options."""

        model = BiologicallyDerivedProductDispensePerformer
        exclude = ["created_at", "updated_at"]


class BiologicallyDerivedProductDispenseSerializer(BaseWritableNestedModelSerializer):
    """biologically derived product dispense serializer."""

    identifier = IdentifierSerializer(required=False, many=True)
    based_on = ServiceRequestReferenceSerializer(required=False, many=True)
    part_of = BiologicallyDerivedProductDispenseReferenceSerializer(
        required=False, many=True
    )
    original_relationship = CodeableConceptSerializer(required=False)
    product = BiologicallyDerivedProductDispenseReferenceSerializer(required=False)
    patient = PatientReferenceSerializer(required=False)
    performer = BiologicallyDerivedProductDispensePerformerSerializer(
        required=False, many=True
    )
    location = LocationReferenceSerializer(required=False)
    quantity = SimpleQuantitySerializer(required=False)
    destination = LocationReferenceSerializer(required=False)
    note = AnnotationSerializer(required=False)

    class Meta:
        """meta options."""

        model = BiologicallyDerivedProductDispense
        exclude = ["created_at", "updated_at"]



"""observation definitions models."""

from django.db import models

from dfhir.base.models import BaseReference, Identifier, TimeStampedModel

from .choices import (
    ObservationDefinitionPermittedDataTypeChoices,
    ObservationDefinitionStatusChoices,
    QualifiedValueGenderChoices,
    QualifiedValueRangeCategoryChoices,
)


class ObservationDefinitionQualifiedValue(TimeStampedModel):
    """observation definition quality value model."""

    context = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_definition_qualified_value_context",
    )
    applies_to = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_definition_qualified_value_applies_to",
    )
    gender = models.CharField(
        max_length=255, null=True, choices=QualifiedValueGenderChoices.choices
    )
    age = models.ForeignKey(
        "base.Range",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_definition_qualified_value_age",
    )
    gestational_age = models.ForeignKey(
        "base.Range",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_definition_qualified_value_gestational_age",
    )
    condition = models.CharField(max_length=255, null=True)
    range_category = models.CharField(
        max_length=255, null=True, choices=QualifiedValueRangeCategoryChoices.choices
    )
    range = models.ForeignKey(
        "base.Range",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_definition_qualified_value_range",
    )
    # TODO: valid_coded_valueset = models.ForeignKey(
    #     "Valueset", on_delete=models.SET_NULL, null=True
    # )
    # TODO: normal_coded_valueset = models.ForeignKey(
    #     "Valueset", on_delete=models.SET_NULL, null=True
    # )
    # TODO: abnormal_coded_valueset = models.ForeignKey(
    #     "Valueset", on_delete=models.SET_NULL, null=True
    # )
    # TODO: critical_coded_valueset = models.ForeignKey(
    #     "Valueset", null=True, on_delete=models.SET_NULL
    # )
    interpretation = models.ManyToManyField(
        "base.CodeableConcept",
        blank=True,
        related_name="observation_definition_qualified_value_interpretation",
    )


class ObservationDefinitionComponent(TimeStampedModel):
    """observation definition component model."""

    code = models.ForeignKey(
        "base.CodeableConcept", null=True, on_delete=models.SET_NULL
    )
    permitted_data_type = models.CharField(
        max_length=255,
        null=True,
        choices=ObservationDefinitionPermittedDataTypeChoices.choices,
    )
    permitted_unit = models.ManyToManyField(
        "base.Coding",
        blank=True,
        related_name="observation_definition_component_permitted_unit",
    )
    qulaified_value = models.ManyToManyField(
        ObservationDefinitionQualifiedValue,
        blank=True,
        related_name="observation_definition_component_qualified_value",
    )


class ObservationDefinitionQuestionnaireReference(BaseReference):
    """observation definition questionnaire reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_definition_questionnaire_reference_identifier",
    )
    # TODO: questionnaire = models.ForeignKey(
    #     "questionnaires.Questionnaire",
    #     on_delete=models.SET_NULL,
    #     null=True,
    #     related_name="observation_definition_questionnaire_reference_questionnaire",
    # )
    observatoin_definition = models.ForeignKey(
        "ObservationDefinition",
        on_delete=models.SET_NULL,
        null=True,
    )


class ObservationDefinition(TimeStampedModel):
    """observation definition model."""

    url = models.URLField(null=True)
    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_definition_identifier",
    )
    version = models.CharField(max_length=255, null=True)
    version_algorithm_string = models.CharField(max_length=255, null=True)
    version_algorithm_coding = models.ForeignKey(
        "base.Coding",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_definition_version_algorithm_coding",
    )
    name = models.CharField(max_length=255, null=True)
    title = models.CharField(max_length=255, null=True)
    status = models.CharField(
        max_length=255, null=True, choices=ObservationDefinitionStatusChoices.choices
    )
    experimental = models.BooleanField(default=False)
    date = models.DateTimeField(null=True)
    publisher = models.CharField(max_length=255, null=True)
    contact = models.ManyToManyField(
        "base.ContactDetail", blank=True, related_name="observation_definition_contact"
    )
    description = models.TextField(null=True)
    use_context = models.ManyToManyField(
        "base.UsageContext",
        blank=True,
        related_name="observation_definition_use_context",
    )
    jurisdiction = models.ManyToManyField(
        "base.CodeableConcept",
        blank=True,
        related_name="observation_definition_jurisdiction",
    )
    purpose = models.TextField(null=True)
    copyright = models.TextField(null=True)
    copyright_label = models.CharField(max_length=255, null=True)
    approval_date = models.DateTimeField(null=True)
    last_review_date = models.DateTimeField(null=True)
    effective_period = models.ForeignKey(
        "base.Period",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_definition_effective_period",
    )
    # TODO: derived_from_canonical = models.ManyToManyField("ObservationDefinition", blank=True)
    derived_from_uri = models.URLField(null=True)
    subject = models.ManyToManyField(
        "base.CodeableConcept",
        blank=True,
        related_name="observation_definition_subject",
    )
    performer_type = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_definition_performer_type",
    )
    category = models.ManyToManyField(
        "base.CodeableConcept",
        blank=True,
        related_name="observation_definition_category",
    )
    code = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_definition_code",
    )
    permitted_data_type = models.CharField(
        max_length=255,
        null=True,
        choices=ObservationDefinitionPermittedDataTypeChoices.choices,
    )
    multiple_results_allowed = models.BooleanField(default=False)
    body_site = models.ManyToManyField(
        "base.CodeableConcept",
        blank=True,
        related_name="observation_definition_body_site",
    )
    method = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_definition_method",
    )
    specimen = models.ForeignKey(
        "specimendefinitions.SpecimenDefinition",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_definition_specimen",
    )
    device = models.ForeignKey(
        "devicedefinitions.DeviceDefinition",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_definition_device",
    )
    preferred_report_name = models.CharField(max_length=255, null=True)
    permitted_unit = models.ManyToManyField(
        "base.Coding",
        blank=True,
        related_name="observation_definition_permitted_unit",
    )
    qualified_value = models.ManyToManyField(
        ObservationDefinitionQualifiedValue,
        related_name="observation_definition_qualified_value",
        blank=True,
    )
    has_member = models.ManyToManyField(
        ObservationDefinitionQuestionnaireReference,
        blank=True,
        related_name="observation_definition_has_member",
    )
    component = models.ManyToManyField(
        ObservationDefinitionComponent,
        blank=True,
        related_name="observation_definition_component",
    )


class ObservationDefinitionReference(BaseReference):
    """observation definition reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_definition_reference_identifier",
    )
    observation_definition = models.ForeignKey(
        "ObservationDefinition",
        on_delete=models.SET_NULL,
        null=True,
        related_name="observation_definition_reference_observation_definition",
    )



"""Detected issues choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class DetectedIssueStatusChoices(models.TextChoices):
    """Detected issue status choices."""

    PRELIMINARY = "preliminary", _("Preliminary")
    FINAL = "final", _("Final")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")
    MITIGATED = "mitigated", _("Mitigated")


class DetectedIssueSeverityChoices(models.TextChoices):
    """Detected issue severity choices."""

    HIGH = "high", _("High")
    MODERATE = "moderate", _("Moderate")
    LOW = "low", _("Low")



"""document reference model choices."""

from django.db.models import TextChoices
from django.utils.translation import gettext_lazy as _


class DocumentReferenceStatusChoices(TextChoices):
    """document reference status choices."""

    CURRENT = "current", _("Current")
    SUPPRESSED = "suppressed", _("Suppressed")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in error")


class DocumentReferenceDocStatusChoices(TextChoices):
    """document reference doc status choices."""

    REGISTERED = "registered", _("Registered")
    PARTIAL = "partial", _("Partial")
    PRELIMINARY = "preliminary", _("Preliminary")
    FINAL = "final", _("Final")
    AMENDED = "amended", _("Amended")
    CORRECTED = "corrected", _("Corrected")
    APPENDED = "appended", _("Appended")
    CANCELLED = "cancelled", _("Cancelled")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in error")
    DEPRECATED = "deprecated", _("Deprecated")
    UNKNOWN = "unknown", _("Unknown")



# Generated by Django 5.2 on 2025-04-23 12:46

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='AllergyIntolerance',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('category', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(choices=[('food', 'Food'), ('medication', 'Medication'), ('environment', 'Environment'), ('biologic', 'Biologic')], max_length=255), null=True, size=None)),
                ('criticality', models.CharField(choices=[('low', 'Low'), ('high', 'High'), ('unable-to-assess', 'Unable to Assess')], max_length=255, null=True)),
                ('onset_date_time', models.DateTimeField(null=True)),
                ('onset_string', models.CharField(max_length=255, null=True)),
                ('recorded_date', models.DateTimeField(null=True)),
                ('last_reaction_occurrence', models.DateTimeField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AllergyIntoleranceAsserterReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AllergyIntoleranceReaction',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('description', models.TextField(null=True)),
                ('onset', models.DateTimeField(null=True)),
                ('severity', models.CharField(choices=[('mild', 'Mild'), ('moderate', 'Moderate'), ('severe', 'Severe')], max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='AllergyIntoleranceRecorderReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""Invoice choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class InvoiceStatus(models.TextChoices):
    """Invoice status choices."""

    DRAFT = "draft", _("Draft")
    ISSUED = "issued", _("Issued")
    BALANCED = "balanced", _("Balanced")
    CANCELLED = "cancelled", _("Cancelled")
    ENTERED_IN_ERROR = "entered_in_error", _("Entered in error")



"""Device usage choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class DeviceUsageStatus(models.TextChoices):
    """Device usage status choices."""

    ACTIVE = "active", _("Active")
    COMPLETED = "completed", _("Completed")
    NOT_DONE = "not-done", _("Not Done")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")
    INTENDED = "intended", _("Intended")
    STOPPED = "stopped", _("Stopped")
    ON_HOLD = "on-hold", _("On Hold")



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('immunizationrecommendations', '0001_initial'),
        ('patients', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='immunizationrecommendation',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_recommendation_patient', to='patients.patientreference'),
        ),
        migrations.AddField(
            model_name='immunizationrecommendationrecommendation',
            name='contraindicated_vaccine_code',
            field=models.ManyToManyField(blank=True, related_name='immunization_recommendation_recommendation_contraindicated_vaccine_code', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='immunizationrecommendationrecommendation',
            name='forecast_reason',
            field=models.ManyToManyField(blank=True, related_name='immunization_recommendation_recommendation_forecast_reason', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='immunizationrecommendationrecommendation',
            name='forecast_status',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_recommendation_recommendation_forecast_status', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='immunizationrecommendationrecommendation',
            name='supporting_patient_information',
            field=models.ManyToManyField(blank=True, related_name='immunization_recommendation_recommendation_supporting_patient_information', to='base.reference'),
        ),
        migrations.AddField(
            model_name='immunizationrecommendationrecommendation',
            name='target_disease',
            field=models.ManyToManyField(blank=True, related_name='immunization_recommendation_recommendation_target_disease', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='immunizationrecommendationrecommendation',
            name='vaccine_code',
            field=models.ManyToManyField(blank=True, related_name='immunization_recommendation_recommendation_vaccine_code', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='immunizationrecommendation',
            name='recommendation',
            field=models.ManyToManyField(blank=True, related_name='immunization_recommendation_recommendation', to='immunizationrecommendations.immunizationrecommendationrecommendation'),
        ),
        migrations.AddField(
            model_name='immunizationrecommendationrecommendationdatecriterion',
            name='code',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_recommendation_recommendation_date_criterion_code', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='immunizationrecommendationrecommendation',
            name='date_criterion',
            field=models.ManyToManyField(blank=True, related_name='immunization_recommendation_recommendation_date_criterion', to='immunizationrecommendations.immunizationrecommendationrecommendationdatecriterion'),
        ),
        migrations.AddField(
            model_name='immunizationrecommendationsupportingimmunizationreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='immunization_recommendation_supporting_immunization_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='immunizationrecommendationrecommendation',
            name='supporting_information',
            field=models.ManyToManyField(blank=True, related_name='immunization_recommendation_supporting_information', to='immunizationrecommendations.immunizationrecommendationsupportingimmunizationreference'),
        ),
    ]



"""Care Teams models."""

from django.db import models

from dfhir.base.models import (
    Annotation,
    BaseReference,
    CodeableConcept,
    ContactPoint,
    Identifier,
    OrganizationReference,
    Period,
    TimeStampedModel,
)
from dfhir.patients.models import PatientGroupReference

from . import choices


class CareTeamParticipantOnBehalfOfReference(BaseReference):
    """Care Team Participant On Behalf Of Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        related_name="care_team_participant_on_behalf_of_reference_identifier",
        on_delete=models.CASCADE,
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner", on_delete=models.CASCADE, null=True
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole", on_delete=models.CASCADE, null=True
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson", on_delete=models.CASCADE, null=True
    )
    patient = models.ForeignKey("patients.Patient", on_delete=models.CASCADE, null=True)
    organization = models.ForeignKey(
        OrganizationReference,
        related_name="care_team_participant_on_behalf_of_reference_organization",
        on_delete=models.CASCADE,
        null=True,
    )
    care_team = models.ForeignKey("CareTeam", on_delete=models.CASCADE, null=True)


class CareTeamParticipantMemberReference(CareTeamParticipantOnBehalfOfReference):
    """Care Team Participant Member Reference model."""


class CareTeamParticipant(TimeStampedModel):
    """Care Team Participant model."""

    role = models.ForeignKey(
        CodeableConcept,
        related_name="care_team_participants_role",
        on_delete=models.CASCADE,
        null=True,
    )
    member = models.ForeignKey(
        CareTeamParticipantMemberReference,
        related_name="care_team_participants_member",
        on_delete=models.CASCADE,
        null=True,
    )
    on_behalf_of = models.ForeignKey(
        CareTeamParticipantOnBehalfOfReference,
        related_name="care_team_participants_on_behalf_of",
        on_delete=models.CASCADE,
        null=True,
    )
    effective_period = models.ForeignKey(
        Period,
        related_name="care_team_participants_effective_period",
        on_delete=models.CASCADE,
        null=True,
    )
    effective_timing = models.ForeignKey(
        "base.Timing",
        related_name="care_team_participants_effective_timing",
        on_delete=models.CASCADE,
        null=True,
    )


class CareTeam(TimeStampedModel):
    """Care Team model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="care_teams_identifier", blank=True
    )
    status = models.CharField(
        max_length=255, choices=choices.CareTeamStatusChoices.choices, null=True
    )
    category = models.ManyToManyField(
        CodeableConcept, related_name="care_teams_category", blank=True
    )
    name = models.CharField(max_length=255, null=True)
    subject = models.ForeignKey(
        PatientGroupReference,
        related_name="care_teams_subject",
        on_delete=models.CASCADE,
        null=True,
    )
    period = models.ForeignKey(
        Period, related_name="care_teams_period", on_delete=models.CASCADE, null=True
    )
    participant = models.ManyToManyField(
        CareTeamParticipant, related_name="care_teams_participant", blank=True
    )
    reason = models.ManyToManyField(
        "conditions.ConditionCodeableReference",
        related_name="care_teams_reason_code",
        blank=True,
    )
    managing_organization = models.ManyToManyField(
        OrganizationReference,
        related_name="care_teams_managing_organization",
        blank=True,
    )
    telecom = models.ManyToManyField(
        ContactPoint, related_name="care_teams_telecom", blank=True
    )
    note = models.ManyToManyField(
        Annotation, related_name="care_teams_note", blank=True
    )


class CareTeamReference(BaseReference):
    """Care Team reference model."""

    identifier = models.ForeignKey(
        Identifier,
        related_name="care_team_reference_identifier",
        on_delete=models.CASCADE,
        null=True,
    )
    care_team = models.ForeignKey(
        CareTeam,
        related_name="care_team_reference_care_team",
        on_delete=models.CASCADE,
        null=True,
    )



"""Service request choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class ServiceRequestStatus(models.TextChoices):
    """Service request status choices."""

    DRAFT = "draft", _("Draft")
    ACTIVE = "active", _("Active")
    ON_HOLD = "on_hold", _("On Hold")
    REVOKED = "revoked", _("Revoked")
    COMPLETED = "completed", _("Completed")
    ENTERED_IN_ERROR = "entered_in_error", _("Entered in Error")
    UNKNOWN = "unknown", _("Unknown")


class ServiceRequestIntent(models.TextChoices):
    """Service request intent choices."""

    PROPOSAL = "proposal", _("Proposal")
    PLAN = "plan", _("Plan")
    ORDER = "order", _("Order")
    ORIGINAL_ORDER = "original-order", _("Original Order")
    REFLEX_ORDER = "reflex-order", _("Reflex Order")
    FILLER_ORDER = "filler-order", _("Filler Order")
    INSTANCE_ORDER = "instance-order", _("Instance Order")
    OPTION = "option", _("Option")


class ServiceRequestPriority(models.TextChoices):
    """SErvice request priority choices."""

    ROUTINE = "routine", _("Routine")
    URGENT = "urgent", _("Urgent")
    ASAP = "asap", _("ASAP")
    STAT = "stat", _("Stat")


class ParameterCode(models.TextChoices):
    """parameter code choices."""

    CATHETER_INSERTION = "catheter-insertion", _("Catheter Insertion")
    BODY_ELEVATION = "body-elevation", _("Body Elevation")
    DECVICE_CONFIGURATION = "device-configuration", _("Device Configuration")
    DEVICE_SETTINGS = "device-settings", _("Device Settings")



"""medication dispenses urls."""

from django.urls import path

from dfhir.medicationdispenses.views import (
    MedicationDispenseDetailView,
    MedicationDispenseListView,
)

app_name = "medicationdispenses"

urlpatterns = [
    path(
        "medicationdispenses/",
        MedicationDispenseListView.as_view(),
        name="medicationdispense-list",
    ),
    path(
        "medicationdispenses/<int:pk>/",
        MedicationDispenseDetailView.as_view(),
        name="medicationdispense-detail",
    ),
]



# Generated by Django 5.2 on 2025-05-06 22:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("activitydefinitions", "0010_activityplandefinitionreference"),
    ]

    operations = [
        migrations.AlterField(
            model_name="activitydefinition",
            name="kind",
            field=models.CharField(
                choices=[
                    ("appointment", "Appointment"),
                    ("care-plan", "Care Plan"),
                    ("claim", "Claim"),
                    ("communication-request", "CommunicationRequest"),
                    ("coverage-eligibility-request", "CoverageEligibilityRequest"),
                    ("device-request", "DeviceRequest"),
                    ("enrollment-request", "EnrollmentRequest"),
                    ("immunization-recommendation", "ImmunizationRecommendation"),
                    ("medication-request", "Medication Request"),
                    ("nutrition-order", "Nutrition Order"),
                    ("request-orchestration", "Request Orchestration"),
                    ("service-request", "Service Request"),
                    ("supply-request", "Supply Request"),
                    ("task", "Task"),
                    ("transport", "Transport"),
                    ("vision-prescription", "Vision Prescription"),
                ],
                max_length=255,
                null=True,
            ),
        ),
    ]



"""organization affiliation views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.organizationaffiliations.models import OrganizationAffiliation
from dfhir.organizationaffiliations.serializers import (
    OrganizationAffiliationSerializer,
)


class OrganizationAffiliationListview(APIView):
    """organization affiliation list view."""

    permission_classes = [AllowAny]

    @extend_schema(
        responses={200: OrganizationAffiliationSerializer(many=True)},
    )
    def get(self, request):
        """Get organization affiliations."""
        organization_affiliations = OrganizationAffiliation.objects.all()
        serializer = OrganizationAffiliationSerializer(
            organization_affiliations, many=True
        )
        return Response(serializer.data)

    @extend_schema(
        request=OrganizationAffiliationSerializer,
        responses={201: OrganizationAffiliationSerializer},
    )
    def post(self, request):
        """Create organization affiliation."""
        serializer = OrganizationAffiliationSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class OrganizationAffiliationDetailview(APIView):
    """organization affiliation detail view."""

    def get_object(self, pk):
        """Get organization affiliation object."""
        try:
            return OrganizationAffiliation.objects.get(pk=pk)
        except OrganizationAffiliation.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: OrganizationAffiliationSerializer})
    def get(self, request, pk=None):
        """Get an organization affiliation by PK."""
        organization_affiliation = self.get_object(pk)
        serializer = OrganizationAffiliationSerializer(organization_affiliation)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={200: OrganizationAffiliationSerializer})
    def patch(self, request, pk=None):
        """Update an organization affiliation object."""
        queryset = self.get_object(pk)
        serializer = OrganizationAffiliationSerializer(
            queryset, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    def delete(self, request, pk=None):
        """Delete an organization affiliation object."""
        queryset = self.get_object(pk)
        queryset.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""Family member history urls."""

from django.urls import path

from . import views

app_name = "familymemberhistories"

# TODO: we should consider <uuid:pk> instead of <int:pk> for the primary key of all routes.

urlpatterns = [
    path(
        "familymemberhistories/",
        views.FamilyMemberHistoryListView.as_view(),
        name="familymemberhistory-list",
    ),
    path(
        "familymemberhistories/<int:pk>/",
        views.FamilyMemberHistoryDetailView.as_view(),
        name="familymemberhistory-detail",
    ),
]



"""Payment Notices models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    Money,
    OrganizationReference,
    TimeStampedModel,
)

from . import choices


class PaymentNoticeReporterReference(BaseReference):
    """Payment Notice Reporter Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="payment_notice_reporter_reference",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="payment_notice_reporter_reference",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="payment_notice_reporter_reference",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.CASCADE,
        related_name="payment_notice_reporter_reference",
        null=True,
    )


class PaymentNoticePayeeReference(BaseReference):
    """Payment Notice Payee Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="payment_notice_payee_reference",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.CASCADE,
        related_name="payment_notice_payee_reference",
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.CASCADE,
        related_name="payment_notice_payee_reference",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.CASCADE,
        related_name="payment_notice_payee_reference",
        null=True,
    )


class PaymentNoticeRequestReference(BaseReference):
    """Payment Notice Request Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="payment_notice_request_reference",
        null=True,
    )
    claim = models.ForeignKey(
        "claims.Claim",
        on_delete=models.CASCADE,
        related_name="payment_notice_request_reference",
        null=True,
    )


class PaymentNoticeResponseReference(BaseReference):
    """Payment Notice Response Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="payment_notice_response_reference",
        null=True,
    )
    claim_response = models.ForeignKey(
        "claimresponses.ClaimResponse",
        on_delete=models.CASCADE,
        related_name="payment_notice_response_reference",
        null=True,
    )


class PaymentNotice(TimeStampedModel):
    """Payment Notice model."""

    identifier = models.ManyToManyField(
        Identifier,
        related_name="payment_notice",
        blank=True,
    )
    status = models.CharField(
        max_length=50,
        choices=choices.PaymentNoticeStatus.choices,
        default=choices.PaymentNoticeStatus.ACTIVE,
    )
    request = models.ForeignKey(
        PaymentNoticeRequestReference,
        on_delete=models.CASCADE,
        related_name="payment_notice",
        null=True,
    )
    response = models.ForeignKey(
        PaymentNoticeResponseReference,
        on_delete=models.CASCADE,
        related_name="payment_notice",
        null=True,
    )
    created = models.DateTimeField(auto_created=True, null=True)
    reporter = models.ForeignKey(
        PaymentNoticeReporterReference,
        on_delete=models.CASCADE,
        related_name="payment_notice",
        null=True,
    )
    payment = models.ForeignKey(
        "paymentreconciliations.PaymentReconciliation",
        on_delete=models.CASCADE,
        related_name="payment_notice",
        null=True,
    )
    payment_date = models.DateTimeField(null=True)
    payee = models.ForeignKey(
        PaymentNoticePayeeReference,
        on_delete=models.CASCADE,
        related_name="payment_notice",
        null=True,
    )
    recipient = models.ForeignKey(
        OrganizationReference,
        on_delete=models.CASCADE,
        related_name="payment_notice",
        null=True,
    )
    amount = models.ForeignKey(
        Money,
        on_delete=models.CASCADE,
        related_name="payment_notice",
        null=True,
    )
    payment_status = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="payment_notice",
        null=True,
    )



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('communications', '0001_initial'),
        ('encounters', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='communication',
            name='encounter',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='communication_encounter', to='encounters.encounterreference'),
        ),
        migrations.AddField(
            model_name='communication',
            name='identifier',
            field=models.ManyToManyField(blank=True, related_name='communication_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='communication',
            name='medium',
            field=models.ManyToManyField(blank=True, related_name='communication_medium', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='communication',
            name='note',
            field=models.ManyToManyField(blank=True, related_name='communication_note', to='base.annotation'),
        ),
        migrations.AddField(
            model_name='communication',
            name='part_of',
            field=models.ManyToManyField(blank=True, related_name='communication_part_of', to='base.reference'),
        ),
        migrations.AddField(
            model_name='communication',
            name='priority',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='communication_priority', to='base.coding'),
        ),
        migrations.AddField(
            model_name='communication',
            name='reason',
            field=models.ManyToManyField(blank=True, related_name='communication_reason_code', to='base.codeablereference'),
        ),
        migrations.AddField(
            model_name='communication',
            name='status',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='communication_status', to='base.coding'),
        ),
        migrations.AddField(
            model_name='communication',
            name='status_reason',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='communication_status_reason', to='base.codeableconcept'),
        ),
    ]



"""Accounts views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import Account
from .serializers import AccountSerializer


class AccountListView(APIView):
    """Account list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: AccountSerializer(many=True)})
    def get(self, request):
        """Get a list of accounts."""
        accounts = Account.objects.all()
        serializer = AccountSerializer(accounts, many=True)
        return Response(serializer.data)

    @extend_schema(request=AccountSerializer, responses={201: AccountSerializer})
    def post(self, request):
        """Create an account."""
        serializer = AccountSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class AccountDetailView(APIView):
    """Account detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get an account object."""
        try:
            return Account.objects.get(pk=pk)
        except Account.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: AccountSerializer})
    def get(self, request, pk=None):
        """Get an account."""
        account = self.get_object(pk)
        serializer = AccountSerializer(account)
        return Response(serializer.data)

    @extend_schema(request=AccountSerializer, responses={200: AccountSerializer})
    def patch(self, request, pk=None):
        """Update an account."""
        account = self.get_object(pk)
        serializer = AccountSerializer(account, data=request.data, partial=True)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data)

    @extend_schema(responses={204: None})
    def delete(self, request, pk=None):
        """Delete an account."""
        account = self.get_object(pk)
        account.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""Devices app configuration."""

from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class DevicesConfig(AppConfig):
    """Devices app configuration."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "dfhir.devices"
    verbose_name = _("Devices")
